package com.example.cool_location_flutter_plugin

import android.app.Activity
import androidx.annotation.NonNull
import com.example.my_cool_library.CoolLocation
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel

/** CoolLocationFlutterPlugin */
class CoolLocationFlutterPlugin : FlutterPlugin, ActivityAware {

    private lateinit var channel: MethodChannel
    private lateinit var eventChannel: EventChannel
    private var activity: Activity? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        initMethodChannel(flutterPluginBinding)
        initEventChannel(flutterPluginBinding)
    }

    private fun initEventChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        eventChannel = EventChannel(
            flutterPluginBinding.binaryMessenger,
            "native_channel_id_456")

        eventChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, eventsSink: EventChannel.EventSink) {
                val duration = arguments as? Int ?: 1_000
                CoolLocation.getLocationUpdates(every = duration.toLong()) {
                    activity?.runOnUiThread {
                        eventsSink.success(
                            mapOf(
                                "latitude" to it.latitude,
                                "longitude" to it.longitude
                            ))
                    }
                }
            }

            override fun onCancel(arguments: Any?) {
                CoolLocation.stopPeriodicUpdates()
            }

        })
    }

    private fun initMethodChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(
            flutterPluginBinding.binaryMessenger,
            "native_channel_id_123")

        channel.setMethodCallHandler { call, result ->
            when (call.method) {
                "getCurrentLocation" -> {
                    val (latitude, longitude) = CoolLocation.getCurrentLocation()
                    result.success(mapOf("latitude" to latitude, "longitude" to longitude))
                }
                else -> {
                    result.error("1001", "Unknown operation", null)
                }
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        eventChannel.setStreamHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivity() {
        activity = null
    }
}
