import 'dart:async';

import 'package:flutter/services.dart';

class LatLong {
  final int latitude;
  final int longitude;

  LatLong({
    required this.latitude,
    required this.longitude,
  });
}

class CoolLocationFlutterPlugin {
  static const MethodChannel _locationMethodChannel =
      MethodChannel('native_channel_id_123');

  static const EventChannel _locationEventChannel =
      EventChannel('native_channel_id_456');

  static Future<LatLong?> getCurrentLocation() async {
    final Map<String, int>? currentLocation = await _locationMethodChannel
        .invokeMapMethod<String, int>("getCurrentLocation");

    final latitude = currentLocation?["latitude"];
    final longitude = currentLocation?["longitude"];

    if (latitude != null && longitude != null) {
      return LatLong(latitude: latitude, longitude: longitude);
    } else {
      return null;
    }
  }

  static Stream<LatLong?> startObservingLocation(int duration) =>
      _locationEventChannel.receiveBroadcastStream(duration).map((event) {
        final convertedMap =
            (event as Map<dynamic, dynamic>?)?.cast<String, int>();

        final latitude = convertedMap?["latitude"];
        final longitude = convertedMap?["longitude"];

        if (latitude != null && longitude != null) {
          return LatLong(latitude: latitude, longitude: longitude);
        } else {
          return null;
        }
      });
}
